@@ -19,11 +19,13 @@
 #include <limits>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <tuple>
 #include <unordered_map>
 #include <unordered_set>
+#include <utility>
 #include <variant>
 
 #include "./errorhandling.h"
@@ -81,9 +83,8 @@ inline RAPIDJSON_NAMESPACE::Document parseJsonDocFromString(const char *json, st
 // define traits to distinguish between "built-in" types like int, std::string, std::vector, ... and custom structs/classes
 template <typename Type>
 using IsBuiltInType = Traits::Any<std::is_integral<Type>, std::is_floating_point<Type>, std::is_pointer<Type>, std::is_enum<Type>,
-    Traits::IsSpecializationOf<Type, std::tuple>, Traits::IsSpecializationOf<Type, std::pair>, Traits::IsIteratable<Type>,
-    Traits::IsSpecializationOf<Type, std::unique_ptr>, Traits::IsSpecializationOf<Type, std::shared_ptr>,
-    Traits::IsSpecializationOf<Type, std::weak_ptr>, IsVariant<Type>>;
+    Traits::IsSpecializingAnyOf<Type, std::tuple, std::pair>, Traits::IsIteratable<Type>,
+    Traits::IsSpecializingAnyOf<Type, std::unique_ptr, std::shared_ptr, std::weak_ptr, std::optional>, IsVariant<Type>>;
 template <typename Type> using IsCustomType = Traits::Not<IsBuiltInType<Type>>;
 
 // define trait to check for custom structs/classes which are JSON serializable
@@ -147,12 +148,24 @@ inline void push(const Type &reflectable, RAPIDJSON_NAMESPACE::Value &value, RAP
 /*!
  * \brief Pushes the specified integer/float/boolean to the specified value.
  */
-template <typename Type, Traits::EnableIfAny<std::is_integral<Type>, std::is_floating_point<Type>> * = nullptr>
+template <typename Type,
+    Traits::EnableIfAny<
+        Traits::All<std::is_integral<Type>, Traits::Not<std::is_same<Type, std::uint8_t>>, Traits::Not<std::is_same<Type, std::int8_t>>>,
+        std::is_floating_point<Type>> * = nullptr>
 inline void push(Type reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_NAMESPACE::Document::AllocatorType &allocator)
 {
     value.Set(reflectable, allocator);
 }
 
+/*!
+ * \brief Pushes the specified 8-bit integer to the specified value.
+ */
+template <typename Type, Traits::EnableIfAny<std::is_same<Type, std::uint8_t>, std::is_same<Type, std::int8_t>> * = nullptr>
+inline void push(Type reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_NAMESPACE::Document::AllocatorType &allocator)
+{
+    value.Set(static_cast<int>(reflectable), allocator);
+}
+
 /*!
  * \brief Pushes the specified enumeration item to the specified value.
  */
@@ -177,7 +190,7 @@ inline void push(Type reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_
 }
 
 /*!
- * \brief Pushes the specified C-string to the specified value.
+ * \brief Pushes the specified std::string_view to the specified value.
  */
 template <typename Type, Traits::EnableIf<std::is_same<Type, std::string_view>> * = nullptr>
 inline void push(Type reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_NAMESPACE::Document::AllocatorType &allocator)
@@ -206,7 +219,7 @@ void push(const Type &reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_
 {
     value.SetArray();
     RAPIDJSON_NAMESPACE::Value::Array array(value.GetArray());
-    array.Reserve(reflectable.size(), allocator);
+    array.Reserve(rapidJsonSize(reflectable.size()), allocator);
     for (const auto &item : reflectable) {
         push(item, array, allocator);
     }
@@ -306,11 +319,10 @@ void push(const Type &reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_
 }
 
 /*!
- * \brief Pushes the specified unique_ptr, shared_ptr or weak_ptr to the specified value.
+ * \brief Pushes the specified unique_ptr, shared_ptr, weak_ptr or optional to the specified value.
  */
 template <typename Type,
-    Traits::EnableIfAny<Traits::IsSpecializationOf<Type, std::unique_ptr>, Traits::IsSpecializationOf<Type, std::shared_ptr>,
-        Traits::IsSpecializationOf<Type, std::weak_ptr>> * = nullptr>
+    Traits::EnableIfAny<Traits::IsSpecializingAnyOf<Type, std::unique_ptr, std::shared_ptr, std::weak_ptr, std::optional>> * = nullptr>
 void push(const Type &reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_NAMESPACE::Document::AllocatorType &allocator)
 {
     if (!reflectable) {
@@ -332,7 +344,7 @@ void push(const Type &reflectable, RAPIDJSON_NAMESPACE::Value &value, RAPIDJSON_
     }
 
     RAPIDJSON_NAMESPACE::Value index, data;
-    index.SetInt(reflectable.index());
+    index.SetUint64(reflectable.index());
     std::visit(
         [&data, &allocator](const auto &reflectableOfActualType) {
             if constexpr (!std::is_same_v<std::decay_t<decltype(reflectableOfActualType)>, std::monostate>) {
@@ -479,6 +491,12 @@ void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::
 template <typename Type, Traits::EnableIf<Traits::IsSpecializationOf<Type, std::shared_ptr>> * = nullptr>
 void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::UTF8<char>> &value, JsonDeserializationErrors *errors);
 
+/*!
+ * \brief Pulls the specified \a reflectable which is an std::optional from the specified value which might be null.
+ */
+template <typename Type, Traits::EnableIf<Traits::IsSpecializationOf<Type, std::optional>> * = nullptr>
+void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::UTF8<char>> &value, JsonDeserializationErrors *errors);
+
 /*!
  * \brief Pulls the specified \a reflectable which is a variant from the specified value which might be null.
  */
@@ -511,7 +529,8 @@ void pull(Type &reflectable, const RAPIDJSON_NAMESPACE::GenericValue<RAPIDJSON_N
  * \brief Pulls the integer or float from the specified value which is supposed and checked to contain the right type.
  */
 template <typename Type,
-    Traits::EnableIf<Traits::Not<std::is_same<Type, bool>>, Traits::Any<std::is_integral<Type>, std::is_floating_point<Type>>> * = nullptr>
+    Traits::EnableIf<Traits::Not<std::is_same<Type, bool>>, Traits::Not<std::is_same<Type, std::uint8_t>>,
+        Traits::Not<std::is_same<Type, std::int8_t>>, Traits::Any<std::is_integral<Type>, std::is_floating_point<Type>>> * = nullptr>
 inline void pull(
     Type &reflectable, const RAPIDJSON_NAMESPACE::GenericValue<RAPIDJSON_NAMESPACE::UTF8<char>> &value, JsonDeserializationErrors *errors)
 {
@@ -524,6 +543,20 @@ inline void pull(
     reflectable = value.Is<Type>() ? value.Get<Type>() : static_cast<Type>(value.GetDouble());
 }
 
+/*!
+ * \brief Pulls the integer or float from the specified value which is supposed and checked to contain the right type.
+ */
+template <typename Type, Traits::EnableIfAny<std::is_same<Type, std::uint8_t>, std::is_same<Type, std::int8_t>> * = nullptr>
+inline void pull(
+    Type &reflectable, const RAPIDJSON_NAMESPACE::GenericValue<RAPIDJSON_NAMESPACE::UTF8<char>> &value, JsonDeserializationErrors *errors)
+{
+    int i = 0;
+    pull(i, value, errors);
+    if (value.IsNumber()) {
+        reflectable = static_cast<Type>(i);
+    }
+}
+
 /*!
  * \brief Pulls the boolean from the specified value which is supposed and checked to contain the right type.
  */
@@ -668,7 +701,7 @@ void pull(Type &reflectable, rapidjson::GenericValue<RAPIDJSON_NAMESPACE::UTF8<c
         ++index;
         typename Type::value_type itemObj;
         pull(itemObj, item, errors);
-        reflectable.emplace(move(itemObj));
+        reflectable.emplace(std::move(itemObj));
     }
 
     // clear error context
@@ -696,7 +729,7 @@ void pull(Type &reflectable, rapidjson::GenericValue<RAPIDJSON_NAMESPACE::UTF8<c
         ++index;
         typename Type::value_type itemObj;
         pull(itemObj, item, errors);
-        if (!reflectable.emplace(move(itemObj)).second) {
+        if (!reflectable.emplace(std::move(itemObj)).second) {
             errors->reportUnexpectedDuplicate(JsonType::Array);
         }
     }
@@ -745,9 +778,9 @@ void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::
             continue;
         }
         const auto array = i->value.GetArray();
-        for (const auto &value : array) {
+        for (const auto &arrayValue : array) {
             auto insertedIterator = reflectable.insert(typename Type::value_type(i->name.GetString(), typename Type::mapped_type()));
-            pull(insertedIterator->second, value, errors);
+            pull(insertedIterator->second, arrayValue, errors);
         }
     }
 }
@@ -849,6 +882,19 @@ void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::
     pull(*reflectable, value, errors);
 }
 
+/*!
+ * \brief Pulls the specified \a reflectable which is an std::optional from the specified value which might be null.
+ */
+template <typename Type, Traits::EnableIf<Traits::IsSpecializationOf<Type, std::optional>> *>
+void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::UTF8<char>> &value, JsonDeserializationErrors *errors)
+{
+    if (value.IsNull()) {
+        reflectable.reset();
+    } else {
+        pull(reflectable.emplace(), value, errors);
+    }
+}
+
 /// \cond
 namespace Detail {
 template <typename Variant, std::size_t compiletimeIndex = 0>
@@ -899,20 +945,14 @@ void pull(Type &reflectable, const rapidjson::GenericValue<RAPIDJSON_NAMESPACE::
         return;
     }
     const auto &indexValue = indexIterator->value;
-    if (!indexValue.IsInt()) {
+    if (!indexValue.IsUint64()) {
         if (errors) {
             errors->emplace_back(JsonDeserializationErrorKind::InvalidVariantIndex, JsonType::Number, jsonType(indexValue.GetType()),
                 errors->currentRecord, errors->currentMember, errors->currentIndex);
         }
         return;
     }
-    const auto index = indexValue.GetInt();
-    if (index < 0) {
-        errors->emplace_back(JsonDeserializationErrorKind::InvalidVariantIndex, JsonType::Number, JsonType::Number, errors->currentRecord,
-            errors->currentMember, errors->currentIndex);
-        return;
-    }
-    Detail::assignVariantValueByRuntimeIndex(static_cast<std::size_t>(index), reflectable, dataIterator->value, errors);
+    Detail::assignVariantValueByRuntimeIndex(indexValue.GetUint64(), reflectable, dataIterator->value, errors);
 }
 
 /*!
@@ -941,7 +981,7 @@ inline void pull(Type &reflectable, const char *name, const rapidjson::GenericVa
     }
 
     // set error context for current member
-    const char *previousMember;
+    const char *previousMember = nullptr;
     if (errors) {
         previousMember = errors->currentMember;
         errors->currentMember = name;
@@ -1018,7 +1058,7 @@ RAPIDJSON_NAMESPACE::Document toJsonDocument(const char *reflectable)
 }
 
 /*!
- * \brief Serializes the specified \a reflectable which is a C-string.
+ * \brief Serializes the specified \a reflectable which is an std::string_view.
  */
 template <typename Type, Traits::EnableIf<std::is_same<Type, std::string_view>> * = nullptr>
 RAPIDJSON_NAMESPACE::Document toJsonDocument(std::string_view reflectable)
